# Structure
- `goal_prox` All project specific code. Subfolders described below. 
  - `analysis` Plotting scripts
  - `baselines` Only the dynamical distance baseline here for now. 
  - `cmds` Templated commands for my experiment running system. I will describe
    how to run these below. 
  - `envs` Wrappers to make the environments work with OpenAI gym and provide
    any preprocessing. 
    - `fetch` Scripts to generate demonstrations and additional environment
      wrapper definitions. 
  - `gym_minigrid` Code for grid world environment. 
  - `method` **All the code for our method** is in `goal_prox/method/prox_func.py`. This is probably the **most important file**. 
  - `policies` Nothing here except a hardcoded policy to solve grid world
    environments. 
- `rlf` My reinforcement learning framework. It includes a lot of functionality
  not used in this project  


## Common Commands

Generating expert Atari demonstrations
```
py -m rlf --cfg tests/config.yaml --cmd ppo/atari  --seed "31"  --cd 2 --env-name 'BreakoutNoFrameskip-v4' --load-file ./data/trained_models/BreakoutNoFrameskip-v4/debug/model_9764.pt  --eval-only --eval-save --no-wb --eval-num-processes 20 --num-eval 5
```


## Additional Details About RLF
I will describe some of the important functionality I use from `rlf`.
`EnvInterface` is a class that is registered and then created in the
environment creation process. I can specify additional command line arguments
from this class and transform the environment. Good examples are in
`goal_prox/envs/vizdoom_wrap.py` and `GoalFetchInterface` in
`goal_prox/envs/goal_check.py`. 

Using the experiment manager to automate runs. This is how you would run three
seeds for our method on the Fetch Pick and Place environment:

```
py -m rlf --cmd ours/fetch --cuda False --seed "31,41,51" --sess-id 0
```

This will open three new tmux windows in session 0, one for each seed. You can
also drop the `--sess-id` argument and it will just be run in the current
window. All of the commands I run are defined in `goal_prox/cmds`. This
contains the python command and the arguments that should be run. I do this so
I keep the command line arguments consistent and easily keep track of the
commands. You can also specify additional arguments. For a full list of the
**base** arguments see: `rlf/args.py`. For the arguments relevant to our
**method** see `get_add_args` in `goal_prox/method/prox_func.py`. Anyone can
define this `get_add_args` function. For instance, in `rlf/algos/il/gail.py`
this is also defined to add the GAIL specific arguments. 

# Environments
Every environment must the key `ep_found_goal` in its info dictionary. 

- To plot everything: `py rlf/exp_mgr/auto_plot.py --plot-cfg goal_prox/analysis/plot_cfgs/all.yaml` 
- For a list of the VizDoom tasks see here https://github.com/shakenes/vizdoomgym/blob/master/vizdoomgym/envs/scenarios/README.md 
